/****** gunstick.acepansion/--background-- **********************************
*
* DESCRIPTION
*  This plugin provides Gun-Stick emulation to ACE CPC Emulator.
*
* HISTORY
*
*****************************************************************************
*
*/

#include <clib/alib_protos.h>

#include <proto/exec.h>

#include <acepansion/lib_header.h>
#include <libraries/acepansion_plugin.h>

#include "acepansion.h"

#define CATCOMP_NUMBERS
#include "generated/locale_strings.h" /* catalog string ids */



struct PluginData
{
    // Plugin generic part (do NEVER put anything before this!)
    struct ACEpansionPlugin common;

    // Light gun specifics
    ULONG diodePulse;
    BOOL firePressed;
};



/*
** Private function which is called to initialize commons
** just after the library was loaded into memory and prior
** to any other API call.
**
** This is the good place to open our required libraries.
*/
BOOL InitResources(VOID)
{
    return TRUE;
}



/*
** Private function which is called to free commons
** just before the library is expurged from memory
**
** This is the good place to close our required libraries.
*/
VOID FreeResources(VOID)
{
}



/****** gunstick.acepansion/CreatePlugin ***********************************
*
* NAME
*   CreatePlugin -- (V1)
*
* SYNOPSIS
*   struct ACEpansionPlugin * CreatePlugin(CONST_STRPTR *toolTypes, struct SignalSemaphore *sema)
*
* FUNCTION
*   Create the plugin data structure and configure how it will be handled by
*   ACE. This function is usually called at ACE startup.
*
* INPUTS
*   *toolType
*     A pointer to a NULL terminated array of CONST_STRPTR containing the
*     tooltypes provided with the plugin library.
*   *sema
*     Pointer to a semaphore to use to protect access to custom data from
*     ACEpansionPlugin when accessed from subtasks or within your PrefsWindow.
*
* RESULT
*     A pointer to an allocated and initialized plugin data structure.
*
* SEE ALSO
*   DeletePlugin
*   ActivatePlugin
*
*****************************************************************************
*
*/

struct ACEpansionPlugin * CreatePlugin(UNUSED CONST_STRPTR *toolTypes, UNUSED struct SignalSemaphore *sema)
{
    // We are allocating an extended plugin structure to add our custom stuff
    struct PluginData *plugin = AllocVec(sizeof(struct PluginData), MEMF_PUBLIC|MEMF_CLEAR);

    if(plugin)
    {
        // Initialize plugin specifics
        plugin->diodePulse = 0;
        plugin->firePressed = FALSE;

        // Initialize plugin generic part
        plugin->common.ap_APIVersion     = API_VERSION;
        plugin->common.ap_Flags          = ACE_FLAGSF_ACTIVE_LIGHT_DEVICE
                                         | ACE_FLAGSF_ACTIVE_JOYSTICK
                                         | ACE_FLAGSF_ACTIVE_EMULATE;
        plugin->common.ap_Title          = GetString(MSG_TITLE);
        plugin->common.ap_HelpFileName   = LIBNAME".guide";
        plugin->common.ap_ToggleMenuName = GetString(MSG_MENU_TOGGLE);
        plugin->common.ap_PrefsMenuName  = NULL;
        plugin->common.ap_PrefsWindow    = NULL;
    }

    return (struct ACEpansionPlugin *)plugin;
}



/****** gunstick.acepansion/DeletePlugin ***********************************
*
* NAME
*   DeletePlugin -- (V1)
*
* SYNOPSIS
*   VOID DeletePlugin(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Delete a plugin data structure. This function is usually called when ACE
*   is exiting.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to delete.
*
* RESULT
*
* SEE ALSO
*   CreatePlugin
*   ActivatePlugin
*
*****************************************************************************
*
*/

VOID DeletePlugin(struct ACEpansionPlugin *plugin)
{
    FreeVec(plugin);
}



/****** gunstick.acepansion/ActivatePlugin *********************************
*
* NAME
*   ActivatePlugin -- (V1)
*
* SYNOPSIS
*   BOOL ActivatePlugin(struct ACEpansionPlugin *plugin, BOOL status)
*
* FUNCTION
*   This function is called everytime ACE wants to activate or disactivate your plugin.
*   While you are disactivated, no other function (except Delete) will the called by ACE.
*   Please note that a plugin is always disactivated by ACE prior to DeletePlugin().
*
* INPUTS
*   activate
*     New activation status requested by ACE.
*
* RESULT
*   Actual activation status (you may fail at activation for some reason, but you
*   should never fail at disactivation request).
*
* SEE ALSO
*   CreatePlugin
*   DeletePlugin
*
*****************************************************************************
*
*/

BOOL ActivatePlugin(UNUSED struct ACEpansionPlugin *plugin, BOOL activate)
{
    return activate;
}



/****** gunstick.acepansion/Reset ******************************************
*
* NAME
*   Reset -- (V1)
*
* SYNOPSIS
*   VOID Reset(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function that is called everytime ACE is issuing a bus reset.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*
* RESULT
*
* SEE ALSO
*   Emulate
*
*****************************************************************************
*
*/

VOID Reset(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** gunstick.acepansion/Emulate ****************************************
*
* NAME
*   Emulate -- (V1)
*
* SYNOPSIS
*   VOID Emulate(struct ACEpansionPlugin *plugin, ULONG *signals)
*
* FUNCTION
*   Function that is called at each microsecond of the emulation step inside ACE.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   *signals
*     Pointer to the I/O signals from the expansion port you can use or alter depending
*     on what you are emulating.
*     The signals bits can be decoded using the following flags:
*       ACE_SIGNALF_INT
*       ACE_SIGNALF_NMI
*       ACE_SIGNALF_LPEN
*
* RESULT
*
* SEE ALSO
*   Reset
*
*****************************************************************************
*
*/

VOID Emulate(struct ACEpansionPlugin *plugin, UNUSED ULONG *signals)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(myPlugin->diodePulse)
    {
        myPlugin->diodePulse--;
    }
}



/****** gunstick.acepansion/WriteIO ****************************************
*
* NAME
*   WriteIO -- (V1)
*
* SYNOPSIS
*   VOID WriteIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE value)
*
* FUNCTION
*   Function that is called everytime Z80 is performing a I/O port write operation.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   port
*     Z80 port on with the I/O write operation was issued.
*   value
*     Value that was issued on the port.
*
* RESULT
*
* SEE ALSO
*   ReadIO
*
*****************************************************************************
*
*/

VOID WriteIO(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT port, UNUSED UBYTE value)
{
}



/****** gunstick.acepansion/ReadIO *****************************************
*
* NAME
*   ReadIO -- (V1)
*
* SYNOPSIS
*   VOID ReadIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE *value)
*
* FUNCTION
*   Function that is called everytime Z80 is performing a I/O port read operation.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   port
*     Z80 port on with the I/O read operation was issued.
*   *value
*     Value to be returned.
*
* RESULT
*
* SEE ALSO
*   WriteIO
*
*****************************************************************************
*
*/

VOID ReadIO(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT port, UNUSED UBYTE *value)
{
}



/****** gunstick.acepansion/GetAudioSample *********************************
*
* NAME
*   GetAudioSample -- (V1)
*
* SYNOPSIS
*   VOID GetAudio(struct ACEpansionPlugin *plugin, SHORT *leftSample, SHORT *rightSample)
*
* FUNCTION
*   Function to be used when your plugin needs to mix an audio output with the one
*   from ACE.
*   It is called at each PSG emulation step, which means at 125KHz.
*
* INPUTS
*   *leftSample
*     Pointer to store the left audio sample to be mixed with ACE audio.
*   *rightSample
*     Pointer to store the right audio sample to be mixed with ACE audio.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID GetAudio(UNUSED struct ACEpansionPlugin *plugin, UNUSED SHORT *leftSample, UNUSED SHORT *rightSample)
{
}



/****** gunstick.acepansion/Printer ****************************************
*
* NAME
*   Printer -- (V1)
*
* SYNOPSIS
*   VOID WritePrinter(struct ACEpansionPlugin *plugin, UBYTE data, BOOL strobe, BOOL *busy)
*
* FUNCTION
*   Function that is called everytime the CPC is accessing the printer port.
*
* INPUTS
*   data
*     8 bits written value (note that CPC is limited to 7 bits values, only Amstrad Plus
*     can actually provide the 8th bit).
*   strobe
*     Status of the /STROBE signal from the printer port.
*   *busy
*     Busy signal value that can be altered.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Printer(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE data, UNUSED BOOL strobe, UNUSED BOOL *busy)
{
}



/****** gunstick.acepansion/Cursor *****************************************
*
* NAME
*   Cursor -- (V1)
*
* SYNOPSIS
*   VOID Cursor(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function which is called everytime the cursor signal from the CRTC is
*   active.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Cursor(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** gunstick.acepansion/AcknowledgeInterrupt ***************************
*
* NAME
*   AcknowledgeInterrupt -- (V1)
*
* SYNOPSIS
*   VOID AcknowledgeInterrupt(struct ACEpansionPlugin *plugin, UBYTE *ivr)
*
* FUNCTION
*   Function which is called everytime the Z80 is entering an interrupt.
*
* INPUTS
*   *ivr
*     IVR could be set to the desired value (Interrupt Vector Register).
*
* RESULT
*
* SEE ALSO
*   ReturnInterrupt
*
*****************************************************************************
*
*/

VOID AcknowledgeInterrupt(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE *ivr)
{
}



/****** gunstick.acepansion/Joystick ***************************************
*
* NAME
*   Joystick -- (V2)
*
* SYNOPSIS
*   VOID Joystick(struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData)
*
* FUNCTION
*   Function which is called everytime the CPC is accessing the joystick port.
*   Please note that the joystick port in R/W but you cannot know if the CPC
*   is actually reading or writing to it.
*
* INPUTS
*   com1
*     If com1 is active then the CPC is requesting access to joystick 0.
*   com2
*     If com2 is active then the CPC is requesting access to joystick 1.
*   *ioData
*     If the CPC is reading the joystick port (regular case) ioData will be set
*     to 0xff and you are supposed to set to 0 the bits corresponding to the
*     requested joystick (you should use the ACE_IODATAF_JOYSTICK_XXX defines).
*     Please note the additional ACE_IODATAF_JOYSTICK_PAUSE (which is not
*     existing on CPC) could be used to map the play/pause button of a read
*     joystick  on the 'P' key of the keyboard (which is also the GX4000 pause
*     button).
*     If the CPC is writing the joystick port the ioData will contain the
*     written value to the port and modifying it will have not effect (on a
*     real CPC it could simply destroy the PSG chipset!).
*     You have no reliable way to know if the CPC is actually reading or writing
*     on the joystick port.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Joystick(struct ACEpansionPlugin *plugin, BOOL com1, UNUSED BOOL com2, UBYTE *ioData)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(com1)
    {
        if(myPlugin->firePressed)
        {
            *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE2;
        }
        if(myPlugin->diodePulse)
        {
            *ioData &= ~ACE_IODATAF_JOYSTICK_DOWN;
        }
    }
}



/****** gunstick.acepansion/ReturnInterrupt ********************************
*
* NAME
*   ReturnInterrupt -- (V3)
*
* SYNOPSIS
*   VOID ReturnInterrupt(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function which is called everytime the Z80 is returning from an interrupt
*   using RETI instruction.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*   AcknowledgeInterrupt
*
*****************************************************************************
*
*/

VOID ReturnInterrupt(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** gunstick.acepansion/LightDeviceDiodePulse **************************
*
* NAME
*   LightDeviceDiodePulse -- (V4)
*
* SYNOPSIS
*   VOID LightDeviceDiodePulse(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   This function is called at each photo-diode pulse, when the beam from
*   the screen monitor is just in front of the light device photo-diode.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*   LightDeviceButton
*
*****************************************************************************
*
*/

VOID LightDeviceDiodePulse(struct ACEpansionPlugin *plugin)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    // Photo-diode pulse will stay for 1ms
    myPlugin->diodePulse = 1000;
}



/****** gunstick.acepansion/LightDeviceButton *******************************
*
* NAME
*   LightDeviceButton -- (V4)
*
* SYNOPSIS
*   VOID LightDeviceButton(struct ACEpansionPlugin *plugin, BOOL pressed)
*
* FUNCTION
*   This function is called everytime the state of the light device button
*   is changing.
*
* INPUTS
*   pressed
*     New state of the light device button.
*
* RESULT
*
* NOTE
*   When a light device plugin is activated, ACE is mapping this button on
*   mouse select button (LMB), so that on-screen click will emulate light
*   device button.
*
* SEE ALSO
*   LightDeviceDiodePulse
*
*****************************************************************************
*
*/

VOID LightDeviceButton(struct ACEpansionPlugin *plugin, BOOL pressed)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    myPlugin->firePressed = pressed;
}

