/****** joysticks.acepansion/--background-- *********************************
*
* DESCRIPTION
*  This plugin provides joysticks emulation to ACE CPC Emulator.
*
* HISTORY
*
*****************************************************************************
*
*/

#include <proto/exec.h>
#include <proto/utility.h>
#include <proto/intuition.h>
#include <proto/muimaster.h>

#include <clib/alib_protos.h>

#include <acepansion/lib_header.h>
#include <libraries/acepansion_plugin.h>

#include "acepansion.h"
#define CATCOMP_NUMBERS
#include "generated/locale_strings.h" /* catalog string ids */



struct PluginDataJoy
{
    ULONG idx;
    USHORT buttons;
    BYTE ns;
    BYTE ew;
    BYTE lx;
    BYTE ly;
    BYTE rx;
    BYTE ry;

    STRPTR prefs;
    ULONG analogMode;

    Object *CYA_Select;
};

struct PluginData
{
    // Plugin generic part (do NEVER put anything before this!)
    struct ACEpansionPlugin common;
    // Joysticks specifics
    struct SignalSemaphore *sema;
    ULONG combinedMode;
    struct PluginDataJoy joy0;
    struct PluginDataJoy joy1;
    STRPTR *list;
};



struct Library *UtilityBase;
struct Library *IntuitionBase;
struct Library *MUIMasterBase;



static Object * makeJoyObject();
static Object * makeAnalogObject(ULONG *storage);

static VOID updatePrefs(struct PluginDataJoy *joy, ULONG idx, STRPTR prefs);

static ULONG strLen(CONST_STRPTR str);
static STRPTR strDup(CONST_STRPTR str);

#define TOOLTYPE_JOY0     "JOYSTICK0"
#define TOOLTYPE_JOY1     "JOYSTICK1"
#define TOOLTYPE_ANALOG0  "ANALOG0"
#define TOOLTYPE_ANALOG1  "ANALOG1"
#define TOOLTYPE_COMBINED "COMBINED"

#define METHOD_CONFIGURE_JOY0 0
#define METHOD_CONFIGURE_JOY1 1

struct JoyMsg
{
    struct PluginData *plugin;
    ULONG idx;
};

static VOID GateHook_DoMethodJoystick(VOID);

static struct EmulLibEntry EmulHook_DoMethodJoystick = { TRAP_LIBNR, 0, GateHook_DoMethodJoystick };
static struct Hook Hook_ConfigureJoystick0 = { { NULL, NULL }, (HOOKFUNC)&EmulHook_DoMethodJoystick, NULL, (APTR)METHOD_CONFIGURE_JOY0 };
static struct Hook Hook_ConfigureJoystick1 = { { NULL, NULL }, (HOOKFUNC)&EmulHook_DoMethodJoystick, NULL, (APTR)METHOD_CONFIGURE_JOY1 };



/*
** Private function which is called to initialize commons
** just after the library was loaded into memory and prior
** to any other API call.
**
** This is the good place to open our required libraries.
*/
BOOL InitResources(VOID)
{
    UtilityBase = OpenLibrary("utility.library", 0L);
    IntuitionBase = OpenLibrary("intuition.library", 0L);
    MUIMasterBase = OpenLibrary("muimaster.library", 0L);

    return MUIMasterBase && IntuitionBase && UtilityBase;
}



/*
** Private function which is called to free commons
** just before the library is expurged from memory
**
** This is the good place to close our required libraries.
*/
VOID FreeResources(VOID)
{
    CloseLibrary(MUIMasterBase);
    MUIMasterBase = NULL;

    CloseLibrary(IntuitionBase);
    IntuitionBase = NULL;

    CloseLibrary(UtilityBase);
    UtilityBase = NULL;
}



/****** joysticks.library/CreatePlugin **************************************
*
* NAME
*   CreatePlugin -- (V1) -- GUI
*
* SYNOPSIS
*   struct ACEpansionPlugin * CreatePlugin(CONST_STRPTR *toolTypes, struct SignalSemaphore *sema)
*
* FUNCTION
*   Create the plugin data structure and configure how it will be handled by
*   ACE. This function is usually called at ACE startup.
*
* INPUTS
*   *toolType
*     A pointer to a NULL terminated array of CONST_STRPTR containing the
*     tooltypes provided with the plugin library.
*   *sema
*     Pointer to a semaphore to use to protect access to custom data from
*     ACEpansionPlugin when accessed from subtasks or within your PrefsWindow.
*
* RESULT
*     A pointer to an allocated and initialized plugin data structure.
*
* SEE ALSO
*   DeletePlugin
*   ActivatePlugin
*   GetPrefsPlugin
*
*****************************************************************************
*
*/

struct ACEpansionPlugin * CreatePlugin(CONST_STRPTR *toolTypes, struct SignalSemaphore *sema)
{
    // We are allocating an extended plugin structure to add our custom stuff
    struct PluginData *myPlugin = AllocVec(sizeof(struct PluginData), MEMF_PUBLIC|MEMF_CLEAR);

    if(myPlugin)
    {
        static STRPTR dummy[] = { "...", NULL };

        CONST_STRPTR joy0 = NULL;
        CONST_STRPTR joy1 = NULL;
        BOOL combinedMode = FALSE;
        BOOL joy0AnalogMode = FALSE;
        BOOL joy1AnalogMode = FALSE;

        Object *CHK_Combined;
        Object *CHK_Joy1;

        // Get tooltypes to configure ourself
        if(toolTypes) while(*toolTypes)
        {
            CONST_STRPTR ptr = *toolTypes;
            CONST_STRPTR name = ptr;
            ULONG len = 0;

            if(*ptr != '(')
            {
                while(*ptr != '=' && *ptr != '\0')
                {
                    ptr++;
                    len++;
                }

                if(*ptr == '=')
                {
                    // Parameters
                    if(Strnicmp(name,TOOLTYPE_JOY0,len) == 0)
                    {
                        CONST_STRPTR value = ptr+1;

                        if(*value != '\0')
                        {
                            joy0 = value;
                        }
                    }
                    else if(Strnicmp(name,TOOLTYPE_JOY1,len) == 0)
                    {
                        CONST_STRPTR value = ptr+1;

                        if(*value != '\0')
                        {
                            joy1 = value;
                        }
                    }
                }
                else
                {
                    // Switches
                    if(Strnicmp(name,TOOLTYPE_ANALOG0,len) == 0)
                    {
                        joy0AnalogMode = TRUE;
                    }
                    else if(Strnicmp(name,TOOLTYPE_ANALOG1,len) == 0)
                    {
                        joy1AnalogMode = TRUE;
                    }
                    else if(Strnicmp(name,TOOLTYPE_COMBINED,len) == 0)
                    {
                        combinedMode = TRUE;
                    }
                }
            }
            toolTypes++;
        }

        // Initialize plugin specifics
        myPlugin->sema = sema;

        myPlugin->joy0.prefs = strDup(joy0);
        myPlugin->joy0.analogMode = joy0AnalogMode;

        myPlugin->joy1.prefs = strDup(joy1);
        myPlugin->joy1.analogMode = joy1AnalogMode;

        myPlugin->list = dummy;

        // Initialize plugin generic part
        myPlugin->common.ap_APIVersion     = API_VERSION;
        myPlugin->common.ap_Flags          = ACE_FLAGSF_ACTIVE_JOYSTICK     // We need the joysticks API (but we are not exclusive)
                                           | ACE_FLAGSF_ACTIVE_ANALOG_INPUT // We handle Amstrad Plus analog joysticks
                                           | ACE_FLAGSF_SAVE_PREFS          // We have preferences to save
                                           | ACE_FLAGSF_CATCH_HOST_GAMEPAD; // We use gamepads plugged on the host computer
        myPlugin->common.ap_Title          = GetString(MSG_TITLE);
        myPlugin->common.ap_HelpFileName   = LIBNAME".guide";
        myPlugin->common.ap_ToggleMenuName = GetString(MSG_MENU_TOGGLE);
        myPlugin->common.ap_PrefsMenuName  = GetString(MSG_MENU_PREFS);
        myPlugin->common.ap_PrefsWindow    = VGroup,
            Child, VGroup,
                MUIA_Group_Columns, 2,
                MUIA_ShortHelp, GetString(MSG_GROUP_ASSIGN_HELP),
                GroupFrameT(GetString(MSG_GROUP_ASSIGN_TITLE)),
                Child, Label2(GetString(MSG_LABEL_JOY0)),
                Child, myPlugin->joy0.CYA_Select = makeJoyObject(myPlugin->list),

                Child, HVSpace,
                Child, makeAnalogObject(&myPlugin->joy0.analogMode),

                Child, Label2(GetString(MSG_LABEL_JOY1)),
                Child, myPlugin->joy1.CYA_Select = makeJoyObject(myPlugin->list),

                Child, HVSpace,
                Child, CHK_Joy1 = makeAnalogObject(&myPlugin->joy1.analogMode),
                End,
            Child, HGroup,
                MUIA_ShortHelp, GetString(MSG_GROUP_ADVANCED_HELP),
                GroupFrameT(GetString(MSG_GROUP_ADVANCED_TITLE)),
                Child, HGroup,
                    Child, CHK_Combined = MUI_MakeObject(MUIO_Checkmark, GetString(MSG_LABEL_CHK_COMBINED)),
                    Child, LLabel(GetString(MSG_LABEL_CHK_COMBINED)),
                    Child, HVSpace,
                    MUIA_CycleChain, TRUE,
                    End,
                End,
            End;

        if(myPlugin->common.ap_PrefsWindow)
        {
            // Notify joystick selection
            DoMethod(myPlugin->joy0.CYA_Select, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime,
                     MUIV_Notify_Self, 4, MUIM_CallHook, &Hook_ConfigureJoystick0, myPlugin, MUIV_TriggerValue);
            DoMethod(myPlugin->joy1.CYA_Select, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime,
                     MUIV_Notify_Self, 4, MUIM_CallHook, &Hook_ConfigureJoystick1, myPlugin, MUIV_TriggerValue);
            // Notify combined mode
            DoMethod(CHK_Combined, MUIM_Notify, MUIA_Selected, MUIV_EveryTime,
                     MUIV_Notify_Self, 3, MUIM_WriteLong, MUIV_TriggerValue, &myPlugin->combinedMode);
            DoMethod(CHK_Combined, MUIM_Notify, MUIA_Selected, MUIV_EveryTime,
                     myPlugin->joy1.CYA_Select, 3, MUIM_Set, MUIA_Disabled, MUIV_TriggerValue);
            DoMethod(CHK_Combined, MUIM_Notify, MUIA_Selected, MUIV_EveryTime,
                     CHK_Joy1, 3, MUIM_Set, MUIA_Disabled, MUIV_TriggerValue);
            // Synchronize GUI with prefs
            set(CHK_Combined, MUIA_Selected, combinedMode);
        }
        else
        {
            DeletePlugin((struct ACEpansionPlugin *)myPlugin);
            myPlugin = NULL;
        }
    }

    return (struct ACEpansionPlugin *)myPlugin;
}



/****** joysticks.library/DeletePlugin **************************************
*
* NAME
*   DeletePlugin -- (V1) -- GUI
*
* SYNOPSIS
*   VOID DeletePlugin(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Delete a plugin data structure. This function is usually called when ACE
*   is exiting.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to delete.
*
* RESULT
*
* SEE ALSO
*   CreatePlugin
*   ActivatePlugin
*
*****************************************************************************
*
*/

VOID DeletePlugin(struct ACEpansionPlugin *plugin)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    MUI_DisposeObject(myPlugin->common.ap_PrefsWindow);

    FreeVec(myPlugin->joy0.prefs);
    FreeVec(myPlugin->joy1.prefs);

    FreeVec(myPlugin);
}



/****** joysticks.library/ActivatePlugin ************************************
*
* NAME
*   ActivatePlugin -- (V1) -- GUI
*
* SYNOPSIS
*   BOOL ActivatePlugin(struct ACEpansionPlugin *plugin, BOOL status)
*
* FUNCTION
*   This function is called everytime ACE wants to activate or disactivate your plugin.
*   While you are disactivated, no other function (except Delete) will the called by ACE.
*   Please note that a plugin is always disactivated by ACE prior to DeletePlugin().
*
* INPUTS
*   activate
*     New activation status requested by ACE.
*
* RESULT
*   Actual activation status (you may fail at activation for some reason, but you
*   should never fail at disactivation request).
*
* SEE ALSO
*   CreatePlugin
*   DeletePlugin
*
*****************************************************************************
*
*/

BOOL ActivatePlugin(UNUSED struct ACEpansionPlugin *plugin, BOOL activate)
{
    return activate;
}



/****** joysticks.acepansion/GetPrefsPlugin *********************************
*
* NAME
*   GetPrefsPlugin -- (V5) -- GUI
*
* SYNOPSIS
*   STRPTR * GetPrefsPlugin(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function that is used by ACE to know about the current preferences of the
*   plugins in order to save them.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   pool
*     Memory pool to use to allocate the tooltypes.
*
* RESULT
*   Pointer to a NULL terminated string array containing the tooltypes or NULL
*   when no tooltypes.
*
* SEE ALSO
*   CreatePlugin
*
*****************************************************************************
*
*/

STRPTR * GetPrefsPlugin(struct ACEpansionPlugin *plugin, APTR pool)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;
    STRPTR *tooltypes = AllocPooled(pool, 6*sizeof(STRPTR));

    if(tooltypes)
    {
        LONG i = 0;

        if(myPlugin->joy0.idx)        tooltypes[i++] = MakeToolType(pool, TOOLTYPE_JOY0    , myPlugin->list[myPlugin->joy0.idx]);
        if(myPlugin->joy0.analogMode) tooltypes[i++] = MakeToolType(pool, TOOLTYPE_ANALOG0 , NULL                              );
        if(myPlugin->joy1.idx)        tooltypes[i++] = MakeToolType(pool, TOOLTYPE_JOY1    , myPlugin->list[myPlugin->joy1.idx]);
        if(myPlugin->joy1.analogMode) tooltypes[i++] = MakeToolType(pool, TOOLTYPE_ANALOG1 , NULL                              );
        if(myPlugin->combinedMode)    tooltypes[i++] = MakeToolType(pool, TOOLTYPE_COMBINED, NULL                              );

        tooltypes[i++] = NULL;
    }

    return tooltypes;
}



/****** joysticks.library/Reset *********************************************
*
* NAME
*   Reset -- (V1) -- EMU
*
* SYNOPSIS
*   VOID Reset(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function that is called everytime ACE is issuing a bus reset.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*
* RESULT
*
* SEE ALSO
*   Emulate
*
*****************************************************************************
*
*/

VOID Reset(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** joysticks.library/Emulate *******************************************
*
* NAME
*   Emulate -- (V1) -- EMU
*
* SYNOPSIS
*   VOID Emulate(struct ACEpansionPlugin *plugin, ULONG *signals)
*
* FUNCTION
*   Function that is called at each microsecond of the emulation step inside ACE.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   *signals
*     Pointer to the I/O signals from the expansion port you can use or alter
*     depending on what you are emulating.
*     The signals bits can be decoded using the following flags:
*       ACE_SIGNALF_INT -- (V1)
*         Maskable interrupt; corresponding to /INT signal from CPC expansion
*         port.
*       ACE_SIGNALF_NMI -- (V3)
*         Non-maskable interrupt; corresponding to /NMI signal from CPC
*         expansion port.
*       ACE_SIGNALF_LPEN -- (V4)
*         Light pen trigger; corresponding to /LPEN signal from CPC expansion
*         port.
*       ACE_SIGNALF_BUS_RESET -- (V6)
*         Bus reset trigger; correspond to /BUSRESET signal from CPC expansion
*         port.
*       ACE_SIGNALF_NOT_READY -- (V6)
*         Not ready trigger; correspond to READY signal from CPC expansion port.
*       ACE_SIGNALF_PLUS_BUS -- (V6)
*         Read-only flag indicating that the expansion port is an Amstrad Plus
*         one (i.e. providing a 4.4444 MHz clock instead of 4 MHz).
*
* RESULT
*
* SEE ALSO
*   Reset
*
*****************************************************************************
*
*/

VOID Emulate(UNUSED struct ACEpansionPlugin *plugin, UNUSED ULONG *signals)
{
}



/****** joysticks.library/WriteIO *******************************************
*
* NAME
*   WriteIO -- (V1) -- EMU
*
* SYNOPSIS
*   VOID WriteIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE value)
*
* FUNCTION
*   Function that is called everytime Z80 is performing a I/O port write operation.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   port
*     Z80 port on with the I/O write operation was issued.
*   value
*     Value that was issued on the port.
*
* RESULT
*
* SEE ALSO
*   ReadIO
*
*****************************************************************************
*
*/

VOID WriteIO(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT port, UNUSED UBYTE value)
{
}



/****** joysticks.library/ReadIO ********************************************
*
* NAME
*   ReadIO -- (V1) -- EMU
*
* SYNOPSIS
*   VOID ReadIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE *value)
*
* FUNCTION
*   Function that is called everytime Z80 is performing a I/O port read operation.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   port
*     Z80 port on with the I/O read operation was issued.
*   *value
*     Value to be returned.
*
* RESULT
*
* SEE ALSO
*   WriteIO
*
*****************************************************************************
*
*/

VOID ReadIO(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT port, UNUSED UBYTE *value)
{
}



/****** joystick.acepansion/WriteMem ****************************************

* NAME
*  WriteMem -- (V6) -- EMU
*
* SYNOPSIS
*  BOOL WriteMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE value)
*
* FUNCTION
*  Function that is called everytime Z80 is performing a memory write
*  operation.
*
* INPUTS
*  *plugin
*    Pointer to the plugin data structure to use.
*  address
*    Z80 address on which the memory write operation was issued.
*  value
*    Value that has to be written.
*
* RESULT
*  TRUE if the plugins actually cougth the memory write operation so that
*  internal memory will be left unchanged (/RAMDIS is set to 0).
*
* NOTES
*  Implement this function only if is it really required for your plugin
*  because it is obviously CPU intensive.
*
* SEE ALSO
*  ReadMem()
*
*****************************************************************************
*
*/

BOOL WriteMem(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT address, UNUSED UBYTE value)
{
    return FALSE;
}



/****** joysticks.acepansion/ReadMem ****************************************
*
* NAME
*  ReadMem -- (V6) -- EMU
*
* SYNOPSIS
*  VOID ReadMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE *value, BOOL opcodeFetch) -- (V7)
*  BOOL ReadMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE *value, BOOL opcode) ** DEPRECATED in V7 **

* FUNCTION
*  Function that is called everytime Z80 is performing a memory read
*  operation.
*
* INPUTS
*  *plugin
*    Pointer to the plugin data structure to use.
*  address
*    Z80 address on which memory read operation was issued.
*  *value
*    Value to be returned.
*    Starting with V7, it contains by default the value that might be read by
*    the CPC if the plugin won't catch the memory read, so that a plugin can
*    only snif the bus.
*  opcodeFetch
*    Boolean which is TRUE when the memory read operation is related to
*    Z80 opcode fetching (/M1 signal). If FALSE, then this is a regular
*    read operation during opcode execution.
*
* RESULT
*  ** DEPRECATED in V7 **
*  TRUE if the plugins actually cougth the memory read operation so that
*  internal memory won't be read (both /ROMDIS and /RAMDIS are set to 0).
*
* NOTES
*  Implement this function only if is it really required for your plugin
*  because it is obviously CPU intensive.
*
* SEE ALSO
*  WriteMem()
*
*****************************************************************************
*
*/

VOID ReadMem(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT address, UNUSED UBYTE *value, UNUSED BOOL opcode)
{
}



/****** joysticks.acepansion/GetAudio ***************************************
*
* NAME
*   GetAudio -- (V1) -- EMU
*
* SYNOPSIS
*   VOID GetAudio(struct ACEpansionPlugin *plugin, SHORT *leftSample, SHORT *rightSample)
*
* FUNCTION
*   Function to be used when your plugin needs to mix an audio output with the one
*   from ACE.
*   It is called at each PSG emulation step, which means at 125KHz.
*
* INPUTS
*   *leftSample
*     Pointer to store the left audio sample to be mixed with ACE audio.
*   *rightSample
*     Pointer to store the right audio sample to be mixed with ACE audio.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID GetAudio(UNUSED struct ACEpansionPlugin *plugin, UNUSED SHORT *leftSample, UNUSED SHORT *rightSample)
{
}



/****** joysticks.library/Printer *******************************************
*
* NAME
*   Printer -- (V1) -- EMU
*
* SYNOPSIS
*   VOID WritePrinter(struct ACEpansionPlugin *plugin, UBYTE data, BOOL strobe, BOOL *busy)
*
* FUNCTION
*   Function that is called everytime the CPC is accessing the printer port.
*
* INPUTS
*   data
*     8 bits written value (note that CPC is limited to 7 bits values, only Amstrad Plus
*     can actually provide the 8th bit).
*   strobe
*     Status of the /STROBE signal from the printer port.
*   *busy
*     Busy signal value that can be altered.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Printer(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE data, UNUSED BOOL strobe, UNUSED BOOL *busy)
{
}



/****** joysticks.library/Joystick ******************************************
*
* NAME
*   Joystick -- (V2) -- EMU
*
* SYNOPSIS
*   VOID Joystick(struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData)
*
* FUNCTION
*   Function which is called everytime the CPC is accessing the joystick port.
*   Please note that the joystick port in R/W but you cannot know if the CPC
*   is actually reading or writing to it.
*
* INPUTS
*   com1
*     If com1 is active then the CPC is requesting access to joystick 0.
*   com2
*     If com2 is active then the CPC is requesting access to joystick 1.
*   *ioData
*     If the CPC is reading the joystick port (regular case) ioData will be set
*     to 0xff and you are supposed to set to 0 the bits corresponding to the
*     requested joystick (you should use the ACE_IODATAF_JOYSTICK_XXX defines).
*     Please note the additional ACE_IODATAF_JOYSTICK_PAUSE which is not
*     existing on CPC but that you could use to map the play/pause button of
*     a joystick used in ACE on the 'P' key of the keyboard (or the GX4000
*     pause button).
*     If the CPC is writing the joystick port the ioData will contain the
*     written value to the port and modifying it will have not effect (on a
*     real CPC it could simply destroy the PSG chipset!).
*
* RESULT
    To emulate the analog joystick port of the Amstrad Plus and the GX-4000,
    this function should be used together with AnalogInput().
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Joystick(struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;
    struct PluginDataJoy *joy = NULL;

    if(!myPlugin->combinedMode)
    {
        // This is the legacy mode where one host's gamepad controls one CPC joysticks
        if(com1)
        {
            joy = &myPlugin->joy0;
        }
        else if(com2)
        {
            joy = &myPlugin->joy1;
        }

        if(joy)
        {
            // Buttons mapping
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LEFT)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE1;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_BOTTOM)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE2;
            if(joy->buttons & (ACE_HOSTGAMEPADF_BUTTON_TOP|ACE_HOSTGAMEPADF_BUTTON_LEFT_STICK|ACE_HOSTGAMEPADF_BUTTON_RIGHT_STICK))
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE3;
            // Add a mapping to up direction from a button to jump easily in some games
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_RIGHT)
                *ioData &= ~ACE_IODATAF_JOYSTICK_UP;
            // Pause from gamepad
            if(joy->buttons & (ACE_HOSTGAMEPADF_BUTTON_MIDDLE_LEFT|ACE_HOSTGAMEPADF_BUTTON_MIDDLE_RIGHT))
                *ioData &= ~ACE_IODATAF_JOYSTICK_PAUSE;
            // East-West and L1/R1 are always mapped to left/right, LX and RX only when no analog mode
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_UPPER_REAR_RIGHT || joy->ew > +32 || (!joy->analogMode && (joy->lx > +32 || joy->rx > +32)))
                *ioData &= ~ACE_IODATAF_JOYSTICK_RIGHT;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_UPPER_REAR_LEFT || joy->ew < -32 || (!joy->analogMode && (joy->lx < -32 || joy->rx < -32)))
                *ioData &= ~ACE_IODATAF_JOYSTICK_LEFT;
            // North-South and L2/R2 are always mapped to up/down, LY and RY only when no analog mode
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LOWER_REAR_RIGHT || joy->ns > +32 || (!joy->analogMode && (joy->ly > +32 || joy->ry > +32)))
                *ioData &= ~ACE_IODATAF_JOYSTICK_DOWN;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LOWER_REAR_LEFT || joy->ns <-32 || (!joy->analogMode && (joy->ly < -32 || joy->ry < -32)))
                *ioData &= ~ACE_IODATAF_JOYSTICK_UP;
        }
    }
    else
    {
        // This is the enhanced mode where one host's gamepad controls both CPC joysticks
        joy = &myPlugin->joy0;

        if(com1)
        {
            // Buttons mapping
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LEFT)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE1;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_BOTTOM)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE2;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LEFT_STICK)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE3;
            // Pause from gamepad
            if(joy->buttons & (ACE_HOSTGAMEPADF_BUTTON_MIDDLE_LEFT|ACE_HOSTGAMEPADF_BUTTON_MIDDLE_RIGHT))
                *ioData &= ~ACE_IODATAF_JOYSTICK_PAUSE;
            // East-West are always mapped to joy0 left/right, X only when no analog mode
            if(joy->ew > +32 || (!joy->analogMode && joy->lx > +32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_RIGHT;
            if(joy->ew < -32 || (!joy->analogMode && joy->lx < -32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_LEFT;
            // North-South are always mapped to joy 0 up/down, Y only when no analog mode
            if(joy->ns > +32 || (!joy->analogMode && joy->ly > +32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_DOWN;
            if(joy->ns <-32 || (!joy->analogMode && joy->ly < -32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_UP;
        }
        else if(com2)
        {
            // Buttons mapping
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_TOP)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE1;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_RIGHT)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE2;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_RIGHT_STICK)
                *ioData &= ~ACE_IODATAF_JOYSTICK_FIRE3;
            // Pause from gamepad
            if(joy->buttons & (ACE_HOSTGAMEPADF_BUTTON_MIDDLE_LEFT|ACE_HOSTGAMEPADF_BUTTON_MIDDLE_RIGHT))
                *ioData &= ~ACE_IODATAF_JOYSTICK_PAUSE;
            // L1 and R1 are always mapped to joy1 left/right, Z only when no analog mode
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_UPPER_REAR_RIGHT || (!joy->analogMode && joy->rx > +32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_RIGHT;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_UPPER_REAR_LEFT || (!joy->analogMode && joy->rx < -32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_LEFT;
            // L2 and R2 are always mapped to joy1 up/down, Rot only when no analog mode
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LOWER_REAR_RIGHT || (!joy->analogMode && joy->ry > +32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_DOWN;
            if(joy->buttons & ACE_HOSTGAMEPADF_BUTTON_LOWER_REAR_LEFT || (!joy->analogMode && joy->ry < -32))
                *ioData &= ~ACE_IODATAF_JOYSTICK_UP;
        }
    }
}



/****** joysticks.acepansion/AnalogInput ************************************
*
* NAME
*   AnalogInput -- (V7) -- EMU
*
* SYNOPSIS
*   VOID AnalogInput(struct ACEpansionPlugin *plugin, UBYTE channel, UBYTE *value)
*
* FUNCTION
*   Function which is called everytime the CPC is accessing the analog joystick
*   port of the Amstrad Plus or the GX-4000.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   channel
*     Analog channel number (from 0 to 3).
*   *value
*     Pointer where to store the 6-bit wide value of the data on the channel.
*
* RESULT
*
* NOTES
*   To emulate the analog joystick port of the Amstrad Plus and the GX-4000,
*   this function should be used together with Joystick() which is providing
*   fire buttons (they are shared between both analog and digital joystick
*   ports).
*
* SEE ALSO
*   Joystick()
*
*****************************************************************************
*
*/

VOID AnalogInput(struct ACEpansionPlugin *plugin, UBYTE channel, UBYTE *value)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    switch(channel)
    {
    case 0:
        if(myPlugin->joy0.analogMode)
            *value = (myPlugin->joy0.lx + 128) / 4;
        break;
    case 1:
        if(myPlugin->joy0.analogMode)
            *value = (myPlugin->joy0.ly + 128) / 4;
        break;
    case 2:
        if(myPlugin->joy0.analogMode && myPlugin->combinedMode)
            *value = (myPlugin->joy0.rx + 128) / 4;
        else if(myPlugin->joy1.analogMode && !myPlugin->combinedMode)
            *value = (myPlugin->joy1.lx + 128) / 4;
        break;
    case 3:
        if(myPlugin->joy0.analogMode && myPlugin->combinedMode)
            *value = (myPlugin->joy0.ry + 128) / 4;
        else if(myPlugin->joy1.analogMode && !myPlugin->combinedMode)
            *value = (myPlugin->joy1.ly + 128) / 4;
        break;
    }
}



/****** joysticks.library/AcknowledgeInterrupt ******************************
*
* NAME
*   AcknowledgeInterrupt -- (V1) -- EMU
*
* SYNOPSIS
*   VOID AcknowledgeInterrupt(struct ACEpansionPlugin *plugin, UBYTE *ivr)
*
* FUNCTION
*   Function which is called everytime the Z80 is entering an interrupt.
*
* INPUTS
*   *ivr
*     IVR could be set to the desired value (Interrupt Vector Register).
*
* RESULT
*
* SEE ALSO
*   ReturnInterrupt
*
*****************************************************************************
*
*/

VOID AcknowledgeInterrupt(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE *ivr)
{
}

/****** joysticks.acepansion/ReturnInterrupt ********************************
*
* NAME
*   ReturnInterrupt -- (V3) -- EMU
*
* SYNOPSIS
*   VOID ReturnInterrupt(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function which is called everytime the Z80 is returning from an interrupt
*   using RETI instruction.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*   AcknowledgeInterrupt
*
*****************************************************************************
*
*/

VOID ReturnInterrupt(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** joysticks.acepansion/Cursor *****************************************
*
* NAME
*   Cursor -- (V1) -- EMU
*
* SYNOPSIS
*   VOID Cursor(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function which is called everytime the cursor signal from the CRTC is
*   active.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Cursor(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** joysticks.acepansion/HostGamepadList ********************************
*
* NAME
*   HostGamepadList -- (V7) -- GUI

* SYNOPSIS
*   VOID HostGamepadList(struct ACEpansionPlugin *plugin, STRPTR *list)
*
* FUNCTION
*   This function is called everytime a gamepad in plugged or unplugged from
*   the host so that the plugin can know about the available devices. Index
*   from HostGamepadEvent() will refer to this list.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   list
*     NULL terminated array of strings containing the human readable names of
*     the plugged gamepads, the first one being the localized string for "No
*     joystick". Please note that the contents of this list remains valid
*     until the function is invoked again, so that you can use it safely in
*     your code without duplication.
*
* RESULT
*
* NOTES
*
* SEE ALSO
*   HostGamepadList()
*
*****************************************************************************
*
*/

VOID HostGamepadList(struct ACEpansionPlugin *plugin, STRPTR *list)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;
    ULONG idx;

    myPlugin->list = list;

    myPlugin->joy0.idx = 0;
    myPlugin->joy1.idx = 0;

    for(idx=1; list[idx] != NULL; idx++)
    {
        // Check if we are matching preferences
        if((myPlugin->joy0.idx == 0 && idx == 1) // Default to 1st found joytick
        || (myPlugin->joy0.prefs && Stricmp(myPlugin->joy0.prefs, list[idx]) == 0))
        {
            myPlugin->joy0.idx = idx;
        }
        if((myPlugin->joy1.idx == 0 && idx == 2) // Default to 2nd found joytick
        || (myPlugin->joy1.prefs && Stricmp(myPlugin->joy1.prefs, list[idx]) == 0))
        {
            myPlugin->joy1.idx = idx;
        }
    }

    nnset(myPlugin->joy0.CYA_Select, MUIA_Cycle_Entries, myPlugin->list);
    nnset(myPlugin->joy0.CYA_Select, MUIA_Cycle_Active, myPlugin->joy0.idx);

    nnset(myPlugin->joy1.CYA_Select, MUIA_Cycle_Entries, myPlugin->list);
    nnset(myPlugin->joy1.CYA_Select, MUIA_Cycle_Active, myPlugin->joy1.idx);
}



/****** joysticks.acepansion/HostGamepadEvent *******************************
*
* NAME
*   HostGamepadEvent -- (V7) -- GUI
*
* SYNOPSIS
*   VOID HostGamepadEvent(struct ACEpansionPlugin *plugin, UBYTE index, USHORT buttons, BYTE ns, BYTE ew, BYTE lx, BYTE ly, BYTE rx, BYTE ry)
*
* FUNCTION
*   This function is called everytime the state of a host's gamepad state is
*   changing. It let you handle both analogic and digital devices.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   index
*     Index of the gamepad in gamepad list.
*   buttons
*     State of the gamepad buttons.
*     Buttons bits can be decoded using the ACE_HOSTGAMEPADF_BUTTON_XXX flags.
*   ns
*     Horizontal position of directional buttons from -128 to +127.
*   ew
*     Vertical position of directional buttons from -128 to +127.
*   lx
*     Horizontal position of left stick from -128 to +127.
*   ly
*     Vertical position of left stick from -128 to +127.
*   rx
*     Horizontal position of right stick from -128 to +127.
*   ry
*     Vertical position of right stick from -128 to +127.
*
* RESULT
*
* NOTES
*
* SEE ALSO
*   HostGamepadList()
*   libraries/acepansion_plugin.h
*
*****************************************************************************
*
*/

VOID HostGamepadEvent(struct ACEpansionPlugin *plugin, UBYTE index, USHORT buttons, BYTE ns, BYTE ew, BYTE lx, BYTE ly, BYTE rx, BYTE ry)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(index == myPlugin->joy0.idx)
    {
        myPlugin->joy0.buttons = buttons;
        myPlugin->joy0.ns = ns;
        myPlugin->joy0.ew = ew;
        myPlugin->joy0.lx = lx;
        myPlugin->joy0.ly = ly;
        myPlugin->joy0.rx = rx;
        myPlugin->joy0.ry = ry;
    }
    if(index == myPlugin->joy1.idx)
    {
        myPlugin->joy1.buttons = buttons;
        myPlugin->joy1.ns = ns;
        myPlugin->joy1.ew = ew;
        myPlugin->joy1.lx = lx;
        myPlugin->joy1.ly = ly;
        myPlugin->joy1.rx = rx;
        myPlugin->joy1.ry = ry;
    }
}



/****** joysticks.acepansion/HostMouseEvent *********************************
*
* NAME
*   HostMouseEvent -- (V7) -- GUI

* SYNOPSIS
*   VOID HostMouseEvent(struct ACEpansionPlugin *plugin, UBYTE buttons, SHORT deltaX, SHORT deltaY)

* FUNCTION
*   Function that is called everytime a host's mouse event occured

* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   buttons
*     State of the mouse buttons.
*     The mouse buttons bits can be decoded using the following flags:
*       ACE_HOSTMOUSEF_BUTTON_MAIN
*         State of main mouse button.
*       ACE_HOSTMOUSEF_BUTTON_SECOND
*         State of second mouse button.
*       ACE_HOSTMOUSEF_BUTTON_MIDDLE
*         State of middle mouse button.
*   deltaX
*     Horizontal mouse move until previous event.
*   deltaY
*     Vertical mouse move until previous event.
*
* RESULT
*
* NOTES
*
* SEE ALSO
      libraries/acepansion_plugin.h
*
*****************************************************************************
*
*/

VOID HostMouseEvent(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE buttons, UNUSED SHORT deltaX, UNUSED SHORT deltaY)
{
}



/****** joysticks.acepansion/HostLightDeviceDiodePulse **********************
*
* NAME
*   HostLightDeviceDiodePulse -- (V7) -- EMU
*
* SYNOPSIS
*   VOID HostLightDeviceDiodePulse(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   This function is called at each photo-diode pulse, when the beam from
*   the screen monitor is just in front of the light device photo-diode.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*   HostLightDeviceButton
*
*****************************************************************************
*
*/

VOID HostLightDeviceDiodePulse(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** joysticks.acepansion/HostLightDeviceButton **************************
*
* NAME
*   HostLightDeviceButton -- (V7) -- GUI
*
* SYNOPSIS
*   VOID HostLightDeviceButton(struct ACEpansionPlugin *plugin, BOOL pressed)
*
* FUNCTION
*   This function is called everytime the state of the light device button
*   is changing.
*
* INPUTS
*   pressed
*     New state of the light device button.
*
* RESULT
*
* NOTE
*   When a light device plugin is activated, ACE is mapping this button on
*   mouse main button, so that on-screen click will emulate light device
*   button.
*
* SEE ALSO
*   HostLightDeviceDiodePulse
*
*****************************************************************************
*
*/

VOID HostLightDeviceButton(UNUSED struct ACEpansionPlugin *plugin, UNUSED BOOL pressed)
{
}



/*
** Custom objects creation
*/

static Object * makeJoyObject(STRPTR *list)
{
    return CycleObject,
        MUIA_Cycle_Entries, list,
        MUIA_Cycle_Active, 0,
        MUIA_CycleChain, TRUE,
        End;
}

static Object * makeAnalogObject(ULONG *storage)
{
    Object *CHK_Analog;

    Object *obj = HGroup,
        Child, CHK_Analog = MUI_MakeObject(MUIO_Checkmark, GetString(MSG_CHK_ANALOG)),
        Child, HGroup,
            Child, LLabel(GetString(MSG_CHK_ANALOG)),
            Child, TextObject,
                MUIA_Font, MUIV_Font_Tiny,
                MUIA_Text_Contents, GetString(MSG_AMSTRAD_PLUS_ONLY),
                End,
            Child, HVSpace,
            End,
        MUIA_CycleChain, TRUE,
        End;

    if(obj)
    {
        // Synchronize with prefs
        set(CHK_Analog, MUIA_Selected, *storage);
        // Install notification
        DoMethod(obj, MUIM_Notify, MUIA_Selected, MUIV_EveryTime,
                 MUIV_Notify_Self, 3, MUIM_WriteLong, MUIV_TriggerValue, storage);
    }

    return obj;
}



/*
** Joysticks configuration handling
*/
static VOID updatePrefs(struct PluginDataJoy *joy, ULONG idx, STRPTR prefs)
{
    joy->idx = idx;

    FreeVec(joy->prefs);

    if(idx != 0)
    {
        joy->prefs = strDup(prefs);
    }
    else
    {
        joy->prefs = NULL;
    }
}

static VOID GateHook_DoMethodJoystick(VOID)
{
    struct Hook *hook = (struct Hook *)REG_A0;
    struct JoyMsg *msg = (struct JoyMsg *)REG_A1;
    struct PluginData *myPlugin = msg->plugin;

    ObtainSemaphore(myPlugin->sema);

    switch((ULONG)hook->h_Data)
    {
        case METHOD_CONFIGURE_JOY0:
            updatePrefs(&myPlugin->joy0, msg->idx, myPlugin->list[msg->idx]);
            break;
        case METHOD_CONFIGURE_JOY1:
            updatePrefs(&myPlugin->joy1, msg->idx, myPlugin->list[msg->idx]);
            break;
    }

    ReleaseSemaphore(myPlugin->sema);
}



/*
** Helper functions
*/

static ULONG strLen(CONST_STRPTR str)
{
    ULONG count = 0;

    while(*str++ != '\0') count++;

    return count;
}

static STRPTR strDup(CONST_STRPTR str)
{
    STRPTR newStr = NULL;

    if(str)
    {
        ULONG size = strLen(str)+1;

        if((newStr = AllocVec(size, MEMF_PUBLIC)))
        {
            CopyMem((CONST APTR)str, newStr, size);
        }
    }
    return newStr;
}

