/****** multiface2.acepansion/background ************************************
*
* DESCRIPTION
*  This plugin provides Multiface Two emulation to ACE CPC Emulator.
*
* HISTORY
*
*****************************************************************************
*
*/

#include <clib/alib_protos.h>

#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/utility.h>
#include <proto/muimaster.h>

#include <dos/dos.h>
#include <utility/utility.h>
#include <libraries/asl.h>

#include <acepansion/lib_header.h>
#include <libraries/acepansion_plugin.h>

#include "acepansion.h"
#define CATCOMP_NUMBERS
#include "generated/locale_strings.h" /* catalog string ids */


#define MF2_ROM "Plugins/multiface2.rom"

#define STOP_1ST_STEP 2

struct PluginData
{
    // Plugin generic part (do NEVER put anything before this!)
    struct ACEpansionPlugin common;

    // Multiface Two specifics
    UBYTE   map[0x4000];
    BOOL    isMapped;
    BOOL    isVisible;
    ULONG   stopStep;

    LONG    stopIssued;     // Set from MUI
    LONG    resetIssued;    // Set from MUI
};



struct Library *DOSBase;
struct Library *UtilityBase;
struct Library *IntuitionBase;
struct Library *MUIMasterBase;



static VOID init(struct PluginData *myPlugin);
static Object *makeMF2Button(ULONG c[3], CONST_STRPTR t);

static ULONG red[3]    = { 0xffffffff,0x00000000,0x00000000 };
static ULONG yellow[3] = { 0xffffffff,0xffffffff,0x00000000 };



/*
** Private function which is called to initialize commons
** just after the library was loaded into memory and prior
** to any other API call.
**
** This is the good place to open our required libraries.
*/
BOOL InitResources(VOID)
{
    DOSBase = OpenLibrary(DOSNAME, 0L);
    UtilityBase = OpenLibrary(UTILITYNAME, 0L);
    IntuitionBase = OpenLibrary("intuition.library", 0L);
    MUIMasterBase = OpenLibrary("muimaster.library", 0L);

    return DOSBase != NULL
        && UtilityBase != NULL
        && IntuitionBase != NULL
        && MUIMasterBase != NULL;
}



/*
** Private function which is called to free commons
** just before the library is expurged from memory
**
** This is the good place to close our required libraries.
*/
VOID FreeResources(VOID)
{
    CloseLibrary(DOSBase);
    CloseLibrary(UtilityBase);
    CloseLibrary(IntuitionBase);
    CloseLibrary(MUIMasterBase);
}



/****** multiface2.acepansion/CreatePlugin **********************************
*
* NAME
*   CreatePlugin -- (V1)
*
* SYNOPSIS
*   struct ACEpansionPlugin * CreatePlugin(CONST_STRPTR *toolTypes, struct SignalSemaphore *sema)
*
* FUNCTION
*   Create the plugin data structure and configure how it will be handled by
*   ACE. This function is usually called at ACE startup.
*
* INPUTS
*   *toolType
*     A pointer to a NULL terminated array of CONST_STRPTR containing the
*     tooltypes provided with the plugin library.
*   *sema
*     Pointer to a semaphore to use to protect access to custom data from
*     ACEpansionPlugin when accessed from subtasks or within your PrefsWindow.
*
* RESULT
*   A pointer to an allocated and initialized plugin data structure.
*
* SEE ALSO
*   DeletePlugin
*   ActivatePlugin
*
*****************************************************************************
*
*/

struct ACEpansionPlugin * CreatePlugin(UNUSED CONST_STRPTR *toolTypes, UNUSED struct SignalSemaphore *sema)
{
    // We are allocating an extended plugin structure to add our custom stuff
    struct PluginData *myPlugin = AllocVec(sizeof(struct PluginData), MEMF_PUBLIC|MEMF_CLEAR);

    if(myPlugin)
    {
        Object *BTN_Stop;
        Object *BTN_Reset;

        // Initialize plugin generic part
        myPlugin->common.ap_APIVersion     = API_VERSION;
        myPlugin->common.ap_Flags          = ACE_FLAGSF_ACTIVE_RESET
                                           | ACE_FLAGSF_ACTIVE_EMULATE
                                           | ACE_FLAGSF_ACTIVE_READMEM
                                           | ACE_FLAGSF_ACTIVE_WRITEMEM
                                           | ACE_FLAGSF_ACTIVE_WRITEIO;
        myPlugin->common.ap_Title          = GetString(MSG_TITLE);
        myPlugin->common.ap_HelpFileName   = LIBNAME".guide";
        myPlugin->common.ap_ToggleMenuName = GetString(MSG_MENU_TOGGLE);
        myPlugin->common.ap_PrefsMenuName  = GetString(MSG_MENU_PREFS);
        myPlugin->common.ap_PrefsWindow    = HGroup,
            GroupFrameT(GetString(MSG_CONTROLS)),
            Child, BTN_Stop  = makeMF2Button(red   , "  STOP  " ),
            Child, BTN_Reset = makeMF2Button(yellow, "  RESET  "),
            End;

        // Initialize plugin specifics
        init(myPlugin);

        DoMethod(BTN_Stop,MUIM_Notify,MUIA_Pressed,FALSE,
                 MUIV_Notify_Self,3,MUIM_WriteLong,TRUE,&myPlugin->stopIssued);
        DoMethod(BTN_Reset,MUIM_Notify,MUIA_Pressed,FALSE,
                 MUIV_Notify_Self,3,MUIM_WriteLong,TRUE,&myPlugin->resetIssued);

    }

    return (struct ACEpansionPlugin *)myPlugin;
}



/****** multiface2.acepansion/DeletePlugin **********************************
*
* NAME
*   DeletePlugin -- (V1)
*
* SYNOPSIS
*   VOID DeletePlugin(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Delete a plugin data structure. This function is usually called when ACE
*   is exiting.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to delete.
*
* RESULT
*
* SEE ALSO
*   CreatePlugin
*   ActivatePlugin
*
*****************************************************************************
*
*/

VOID DeletePlugin(struct ACEpansionPlugin *plugin)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    MUI_DisposeObject(myPlugin->common.ap_PrefsWindow);
    FreeVec(myPlugin);
}



/****** multiface2.acepansion/ActivatePlugin ********************************
*
* NAME
*   ActivatePlugin -- (V1)
*
* SYNOPSIS
*   BOOL ActivatePlugin(struct ACEpansionPlugin *plugin, BOOL status)
*
* FUNCTION
*   This function is called everytime ACE wants to activate or disactivate your plugin.
*   While you are disactivated, no other function (except Delete) will the called by ACE.
*   Please note that a plugin is always disactivated by ACE prior to DeletePlugin().
*
* INPUTS
*   activate
*     New activation status requested by ACE.
*
* RESULT
*   Actual activation status (you may fail at activation for some reason, but you
*   should never fail at disactivation request).
*
* SEE ALSO
*   CreatePlugin
*   DeletePlugin
*
*****************************************************************************
*
*/

BOOL ActivatePlugin(struct ACEpansionPlugin *plugin, BOOL activate)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(activate)
    {
        BPTR rom;

        activate = FALSE;

        if((rom = Open(MF2_ROM, MODE_OLDFILE)))
        {
            if(Read(rom, myPlugin->map, 0x2000) == 0x2000)
            {
                activate = TRUE;
            }
            Close(rom);
        }

        if(!activate)
        {
            MUI_Request(NULL, NULL, 0, GetString(MSG_TITLE), GetString(MSG_OK), GetString(MSG_NO_ROM), MF2_ROM);
        }
    }

    init(myPlugin);

    return activate;
}



/****** multiface2.acepansion/GetPrefsPlugin ********************************
*
* NAME
*   GetPrefsPlugin -- (V5)
*
* SYNOPSIS
*   STRPTR * GetPrefsPlugin(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function that is used by ACE to know about the current preferences of the
*   plugins in order to save them.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   pool
*     Memory pool to use to allocate the tooltypes.
*
* RESULT
*   Pointer to a NULL terminated string array containing the tooltypes or NULL
*   when no tooltypes.
*
* SEE ALSO
*   CreatePlugin
*
*****************************************************************************
*
*/

STRPTR * GetPrefsPlugin(UNUSED struct ACEpansionPlugin *plugin, UNUSED APTR pool)
{
    return NULL;
}



/****** multiface2.acepansion/Reset *****************************************
*
* NAME
*   Reset -- (V1)
*
* SYNOPSIS
*   VOID Reset(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function that is called everytime ACE is issuing a bus reset.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*
* RESULT
*
* SEE ALSO
*   Emulate
*
*****************************************************************************
*
*/

VOID Reset(struct ACEpansionPlugin *plugin)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    init(myPlugin);
}


/****** multiface2.acepansion/Emulate ***************************************
*
* NAME
*   Emulate -- (V1)
*
* SYNOPSIS
*   VOID Emulate(struct ACEpansionPlugin *plugin, ULONG *signals)
*
* FUNCTION
*   Function that is called at each microsecond of the emulation step inside ACE.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   *plugin
*     Pointer to the plugin data structure to use.
*   *signals
*     Pointer to the I/O signals from the expansion port you can use or alter
*     depending on what you are emulating.
*     The signals bits can be decoded using the following flags:
*       ACE_SIGNALF_INT -- (V1)
*         Maskable interrupt; corresponding to /INT signal from CPC expansion
*         port.
*       ACE_SIGNALF_NMI -- (V3)
*         Non-maskable interrupt; corresponding to /NMI signal from CPC
*         expansion port.
*       ACE_SIGNALF_LPEN -- (V4)
*         Light pen trigger; corresponding to /LPEN signal from CPC expansion
*         port.
*       ACE_SIGNALF_BUS_RESET -- (V6)
*         Bus reset trigger; correspond to /BUSRESET signal from CPC expansion
*         port.
*       ACE_SIGNALF_NOT_READY -- (V6)
*         Not ready trigger; correspond to READY signal from CPC expansion port.
*       ACE_SIGNALF_PLUS_BUS -- (V6)
*         Read-only flag indicating that the expansion port is an Amstrad Plus
*         one (i.e. providing a 4.4444 MHz clock instead of 4 MHz).
*
* RESULT
*
* SEE ALSO
*   Reset
*
*****************************************************************************
*
*/

VOID Emulate(struct ACEpansionPlugin *plugin, ULONG *signals)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(myPlugin->stopIssued && myPlugin->stopStep == 0)
    {
        *signals |= ACE_SIGNALF_NMI;
    }
    if(myPlugin->resetIssued)
    {
        myPlugin->resetIssued = FALSE;

        *signals |= ACE_SIGNALF_BUS_RESET;
    }
}



/****** multiface2.acepansion/WriteIO ***************************************
*
* NAME
*   WriteIO -- (V1)
*
* SYNOPSIS
*   VOID WriteIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE value)
*
* FUNCTION
*   Function that is called everytime Z80 is performing a I/O port write operation.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   port
*     Z80 port on with the I/O write operation was issued.
*   value
*     Value that was issued on the port.
*
* RESULT
*
* SEE ALSO
*   ReadIO
*
*****************************************************************************
*
*/

VOID WriteIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE value)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if((port & 0xfffc) == 0xfee8)
    {
        if(port & 0x2)
        {
            myPlugin->isMapped = FALSE;
        }
        else if(myPlugin->isVisible)
        {
            myPlugin->isMapped = TRUE;
        }
    }
    else switch(port & 0xff00)
    {
        case 0xf700:
            myPlugin->map[0x37ff] = value;
            break;
        case 0xbc00:
            myPlugin->map[0x3cff] = value;
            break;
        case 0xbd00:
            if((myPlugin->map[0x3cff] & 0xf0) == 0)
            {
                myPlugin->map[0x3db0 + myPlugin->map[0x3cff]] = value;
            }
            break;
        case 0x7f00:
            switch(value & 0xc0)
            {
                case 0x00:
                    myPlugin->map[0x3fcf] = value;
                    break;
                case 0x40:
                    if(myPlugin->map[0x3fcf] & 0x10)
                    {
                        myPlugin->map[0x3fdf] = value;
                    }
                    else
                    {
                        myPlugin->map[0x3f10 + (myPlugin->map[0x3fcf] & 0x0f)] = value;
                        myPlugin->map[0x3f90 + (myPlugin->map[0x3fcf] & 0x0f)] = value;
                    }
                    break;
                case 0x80:
                    myPlugin->map[0x3fef] = value;
                    break;
                case 0xc0:
                    myPlugin->map[0x3fff] = value;
                    break;
            }
            break;
    }
}



/****** multiface2.acepansion/ReadIO ****************************************
*
* NAME
*   ReadIO -- (V1)
*
* SYNOPSIS
*   VOID ReadIO(struct ACEpansionPlugin *plugin, USHORT port, UBYTE *value)
*
* FUNCTION
*   Function that is called everytime Z80 is performing a I/O port read operation.
*   Note: Implement this function only if is it really required for your plugin because it
*   is obviously CPU intensive.
*
* INPUTS
*   port
*     Z80 port on with the I/O read operation was issued.
*   *value
*     Value to be returned.
*
* RESULT
*
* SEE ALSO
*   WriteIO
*
*****************************************************************************
*
*/

VOID ReadIO(UNUSED struct ACEpansionPlugin *plugin, UNUSED USHORT port, UNUSED UBYTE *value)
{
}



/****** multiface2.acepansion/WriteMem **************************************
*
* NAME
*  WriteMem -- (V6)
*
* SYNOPSIS
*  BOOL WriteMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE value)
*
* FUNCTION
*  Function that is called everytime Z80 is performing a memory write
*  operation.
*
* INPUTS
*  *plugin
*    Pointer to the plugin data structure to use.
*  address
*    Z80 address on which the memory write operation was issued.
*  value
*    Value that has to be written.
*
* RESULT
*  TRUE if the plugins actually cougth the memory write operation so that
*  internal memory will be left unchanged (/RAMDIS is set to 0).
*
* NOTES
*  Implement this function only if is it really required for your plugin
*  because it is obviously CPU intensive.
*
* SEE ALSO
*  ReadMem()
*
*****************************************************************************
*
*/

BOOL WriteMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE value)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(myPlugin->isMapped)
    {
        if((address & 0xe000) == 0x2000)
        {
            myPlugin->map[address] = value;
            return TRUE;
        }
    }
    return FALSE;
}



/****** multiface2.acepansion/ReadMem ***************************************

* NAME
*  ReadMem -- (V6)
*
* SYNOPSIS
*  BOOL ReadMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE *value, BOOL opcode)

* FUNCTION
*  Function that is called everytime Z80 is performing a memory read
*  operation.
*
* INPUTS
*  *plugin
*    Pointer to the plugin data structure to use.
*  address
*    Z80 address on which memory read operation was issued.
*  *value
*    Value to be returned.
*  opcode
*    Boolean which is TRUE when the memory read operation is related to
*    Z80 opcode fetching (/M1 signal). If FALSE, then this is a regular
*    read operation during opcode execution.
*
* RESULT
*  TRUE if the plugins actually cougth the memory read operation so that
*  internal memory won't be read (both /ROMDIS and /RAMDIS are set to 0).
*
* NOTES
*  Implement this function only if is it really required for your plugin
*  because it is obviously CPU intensive.
*
* SEE ALSO
*  WriteMem()
*
*****************************************************************************
*
*/

BOOL ReadMem(struct ACEpansionPlugin *plugin, USHORT address, UBYTE *value, BOOL opcode)
{
    struct PluginData *myPlugin = (struct PluginData *)plugin;

    if(myPlugin->stopIssued)
    {
        // Entering Multiface Two stop mode
        // Z80 entering NMI is detected:
        //  - To switch MF2 ROM on
        //  - To make MF2 visible again
        if(opcode && (address == 0x0066 || address == 0x0067))
        {
            myPlugin->stopStep = STOP_1ST_STEP;
            myPlugin->isMapped = TRUE;
            myPlugin->isVisible = TRUE;
        }
        // Handling Multiface Two stop mode
        // &65 is called twice by MF2 ROM:
        //  - When entering stop process (to stop NMI?)
        //  - Just before returning to program (to make MF2 invisible)
        else if(opcode && (address == 0x0064 || address == 0x065))
        {
            if(--myPlugin->stopStep == 0)
            {
                myPlugin->stopIssued = FALSE;
                myPlugin->isVisible = FALSE;
            }
        }
    }

    if(myPlugin->isMapped)
    {
        if((address & 0xc000) == 0x0000)
        {
            *value = myPlugin->map[address];
            return TRUE;
        }
    }
    return FALSE;
}



/****** multiface2.acepansion/GetAudio **************************************
*
* NAME
*   GetAudioSample -- (V1)
*
* SYNOPSIS
*   VOID GetAudio(struct ACEpansionPlugin *plugin, SHORT *leftSample, SHORT *rightSample)
*
* FUNCTION
*   Function to be used when your plugin needs to mix an audio output with the one
*   from ACE.
*   It is called at each PSG emulation step, which means at 125KHz.
*
* INPUTS
*   *leftSample
*     Pointer to store the left audio sample to be mixed with ACE audio.
*   *rightSample
*     Pointer to store the right audio sample to be mixed with ACE audio.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID GetAudio(UNUSED struct ACEpansionPlugin *plugin, UNUSED SHORT *leftSample, UNUSED SHORT *rightSample)
{
}



/****** multiface2.acepansion/Printer ***************************************
*
* NAME
*   Printer -- (V1)
*
* SYNOPSIS
*   VOID WritePrinter(struct ACEpansionPlugin *plugin, UBYTE data, BOOL strobe, BOOL *busy)
*
* FUNCTION
*   Function that is called everytime the CPC is accessing the printer port.
*
* INPUTS
*   data
*     8 bits written value (note that CPC is limited to 7 bits values, only Amstrad Plus
*     can actually provide the 8th bit).
*   strobe
*     Status of the /STROBE signal from the printer port.
*   *busy
*     Busy signal value that can be altered.
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Printer(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE data, UNUSED BOOL strobe, UNUSED BOOL *busy)
{
}



/****** multiface2.acepansion/Joystick **************************************
*
* NAME
*   Joystick -- (V2)
*
* SYNOPSIS
*   VOID Joystick(struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData)
*
* FUNCTION
*   Function which is called everytime the CPC is accessing the joystick port.
*   Please note that the joystick port in R/W but you cannot know if the CPC
*   is actually reading or writing to it.
*
* INPUTS
*   com1
*     If com1 is active then the CPC is requesting access to joystick 0.
*   com2
*     If com2 is active then the CPC is requesting access to joystick 1.
*   *ioData
*     If the CPC is reading the joystick port (regular case) ioData will be set
*     to 0xff and you are supposed to set to 0 the bits corresponding to the
*     requested joystick (you should use the ACE_IODATAF_JOYSTICK_XXX defines).
*     Please note the additional ACE_IODATAF_JOYSTICK_PAUSE which is not
*     existing on CPC but that you could use to map the play/pause button of
*     a joystick used in ACE on the 'P' key of the keyboard (or the GX4000
*     pause button).
*     If the CPC is writing the joystick port the ioData will contain the
*     written value to the port and modifying it will have not effect (on a
*     real CPC it could simply destroy the PSG chipset!).
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Joystick(UNUSED struct ACEpansionPlugin *plugin, UNUSED BOOL com1, UNUSED BOOL com2, UNUSED UBYTE *ioData)
{
}



/****** multiface2.acepansion/LightDeviceDiodePulse *************************
*
* NAME
*   LightDeviceDiodePulse -- (V4)
*
* SYNOPSIS
*   VOID LightDeviceDiodePulse(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   This function is called at each photo-diode pulse, when the beam from
*   the screen monitor is just in front of the light device photo-diode.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*   LightDeviceButton
*
*****************************************************************************
*
*/

VOID LightDeviceDiodePulse(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** multiface2.acepansion/LightDeviceButton *****************************
*
* NAME
*   LightDeviceButton -- (V4)
*
* SYNOPSIS
*   VOID LightDeviceButton(struct ACEpansionPlugin *plugin, BOOL pressed)
*
* FUNCTION
*   This function is called everytime the state of the light device button
*   is changing.
*
* INPUTS
*   pressed
*     New state of the light device button.
*
* RESULT
*
* NOTE
*   When a light device plugin is activated, ACE is mapping this button on
*   mouse select button (LMB), so that on-screen click will emulate light
*   device button.
*
* SEE ALSO
*   LightDeviceDiodePulse
*
*****************************************************************************
*
*/

VOID LightDeviceButton(UNUSED struct ACEpansionPlugin *plugin, UNUSED BOOL pressed)
{
}



/****** multiface2.acepansion/AcknowledgeInterrupt **************************
*
* NAME
*   AcknowledgeInterrupt -- (V1)
*
* SYNOPSIS
*   VOID AcknowledgeInterrupt(struct ACEpansionPlugin *plugin, UBYTE *ivr)
*
* FUNCTION
*   Function which is called everytime the Z80 is entering an interrupt.
*
*   This happens just before the Z80 is actually entering interrupt routine, and
*   it is the good place to provide support for any expansion supporting
*   interrupt vertorized mode (IM 2).
*
* INPUTS
*   *ivr
*     IVR could be set to the desired value (Interrupt Vector Register).
*
* RESULT
*   ReturnInterrupt()
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID AcknowledgeInterrupt(UNUSED struct ACEpansionPlugin *plugin, UNUSED UBYTE *ivr)
{
}



/****** multiface2.acepansion/ReturnInterrupt *******************************
*
* NAME
*   ReturnInterrupt -- (V3)
*
* SYNOPSIS
*   VOID ReturnInterrupt(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function which is called everytime the Z80 is returning from an interrupt
*   using RETI instruction.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*   AcknowledgeInterrupt
*
*****************************************************************************
*
*/

VOID ReturnInterrupt(UNUSED struct ACEpansionPlugin *plugin)
{
}



/****** multiface2.acepansion/Cursor ****************************************
*
* NAME
*   Cursor -- (V1)
*
* SYNOPSIS
*   VOID Cursor(struct ACEpansionPlugin *plugin)
*
* FUNCTION
*   Function which is called everytime the cursor signal from the CRTC is
*   active.
*
* INPUTS
*
* RESULT
*
* SEE ALSO
*
*****************************************************************************
*
*/

VOID Cursor(UNUSED struct ACEpansionPlugin *plugin)
{
}



/* Helper functions */

static VOID init(struct PluginData *myPlugin)
{
    myPlugin->isMapped = FALSE;
    myPlugin->isVisible = TRUE;
    myPlugin->stopStep = 0;

    myPlugin->stopIssued = FALSE;
    myPlugin->resetIssued = FALSE;
}

static Object * makeMF2Button(ULONG c[3], CONST_STRPTR t)
{
    return VGroup,
        ButtonFrame,
        MUIA_Font, MUIV_Font_Button,
        MUIA_Font         , MUIV_Font_Tiny,
        MUIA_InputMode    , MUIV_InputMode_RelVerify,
        MUIA_Background   , MUII_ButtonBack,
        MUIA_CycleChain   , TRUE,
        Child, HVSpace,
        Child, HGroup,
            Child, HVSpace,
            Child, ColorfieldObject,
                MUIA_FixWidthTxt, "OO",
                MUIA_FixHeightTxt, " ",
                MUIA_Colorfield_RGB, c,
                End,
            Child, HVSpace,
            End,
        Child, TextObject,
            MUIA_Text_PreParse, MUIX_C,
            MUIA_Text_Contents, t,
            End,
        Child, HVSpace,
        End;
}
