/*
    ACE expansion plugins definitions

    Copyright  2015-2021 Philippe Rimauro. All rights reserved.
*/

#ifndef LIBRARIES_ACEPANSION_H
#define LIBRARIES_ACEPANSION_H

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif
#ifndef INTUITION_CLASSUSR_H
#include <intuition/classusr.h>
#endif



/*
 * Generic part of the plugin data structure
 */

struct ACEpansionPlugin
{
    ULONG   ap_APIVersion;     /* Version API implemented by the plugin (1 to 7) */
    ULONG   ap_Flags;          /* Flags to handle activate needed emulation APIs and some other options */
                               /* (use combinaison of ACE_FLAGSF_xxx here) */
    STRPTR  ap_Title;          /* Title for the plugin that will be used in the window title and the "Help" menu */
    STRPTR  ap_HelpFileName;   /* Name of the guide to be used from the "Help" menu (NULL if no help) */
    STRPTR  ap_ToggleMenuName; /* Label that will be used for the activate toggle from the "Expansions" menu */
    STRPTR  ap_PrefsMenuName;  /* Label that will be used for the (optional) preferences/control window from the "Expansions" menu */
    Object *ap_PrefsWindow;    /* MUI object that will be used for preferences/control window contents (NULL if none)
                                  Please note that the semaphore provided at CreatePlugin() time shall be used
                                  to access custom part of this data structure from the object code itself
                                  (object and library accessors are not handled by the same task)
                                  Also, don't forget to dispose your object at DeletePlugin() */
};



/*
 * FLAGSF are field definitions to be used to set ap_Flags field in LibCreatePlugin() API
 */

#define ACE_FLAGSF_EXCLUDE_AMSTRAD_PLUS    (1<<25) /* Plugin is not available on Amstrad Plus                                                                ** V4 API only ** */
#define ACE_FLAGSF_EXCLUDE_AMSTRAD_CPC     (1<<24) /* Plugin is not available on Amstrad CPC                                                                 ** V4 API only ** */

#define ACE_FLAGSF_ACTIVE_ANALOG_INPUT     (1<<15) /* Plugin is using Amstrad Plus analog inputs      - AnalogInput()                                        ** V7 API only ** */
#define ACE_FLAGSF_ACTIVE_WRITEMEM         (1<<12) /* Plugin is using memory write                    - WriteMem()                                           ** V6 API only ** */
#define ACE_FLAGSF_ACTIVE_READMEM          (1<<11) /* Plugin is using memory read                     - ReadMem()                                            ** V6 API only ** */
#define ACE_FLAGSF_ACTIVE_RETI             (1<< 9) /* Plugin is using RETI detection                  - ReturnInterrupt()                                    ** V3 API only ** */
#define ACE_FLAGSF_ACTIVE_JOYSTICK         (1<< 8) /* Plugin is using joysticks                       - Joystick()                                           ** V2 API only ** */
#define ACE_FLAGSF_ACTIVE_EMULATE          (1<< 7) /* Plugin is using real-time scheduling            - Emulate()                                                              */
#define ACE_FLAGSF_ACTIVE_RESET            (1<< 6) /* Plugin is using bus reset                       - Reset()                                                                */
#define ACE_FLAGSF_ACTIVE_WRITEIO          (1<< 5) /* Plugin is using I/O ports write                 - WriteIO()                                                              */
#define ACE_FLAGSF_ACTIVE_READIO           (1<< 4) /* Plugin is using I/O port read                   - ReadIO()                                                               */
#define ACE_FLAGSF_ACTIVE_AUDIO            (1<< 3) /* Plugin is using CPC audio output                - GetAudio()                                                             */
#define ACE_FLAGSF_ACTIVE_PRINTER          (1<< 2) /* Plugin is using printer port                    - Printer()                                                              */
#define ACE_FLAGSF_ACTIVE_CURSOR           (1<< 1) /* Plugin is using cursor trigger                  - Cursor()                                                               */
#define ACE_FLAGSF_ACTIVE_ACKINT           (1<< 0) /* Plugin is using interrupt ack trigger           - AcknowledgeInterrupt()                                                 */

#define ACE_FLAGSF_CATCH_HOST_GAMEPAD      (1<<14) /* Plugin is using events from host's gamepads     - HostGamepadList(), HostGamepadEvent()                ** V7 API only ** */
#define ACE_FLAGSF_CATCH_HOST_MOUSE        (1<<13) /* Plugin is using events from host's mouse        - HostMouseEvent()                                     ** V7 API only ** */
#define ACE_FLAGSF_CATCH_HOST_LIGHT_DEVICE (1<<10) /* Plugin is using events from host's light device - HostLightDeviceDiodePulse(), HostLightDeviceButton() ** V7 API only ** */

#define ACE_FLAGSF_SAVE_PREFS              (1<<31) /* Plugin preferences have to be saved by ACE      - GetPrefsPlugin()                                     ** V5 API only ** */



/*
 * SIGNALF are field definitions to be used to alter signals field from plugin LibEmulate() API
 */

#define ACE_SIGNALF_INT       (1<< 0) /* Expansion port /INT signal                                                  */
#define ACE_SIGNALF_NMI       (1<< 1) /* Expansion port /NMI signal                                ** V3 API only ** */
#define ACE_SIGNALF_LPEN      (1<< 2) /* Expansion port /LPEN signal                               ** V4 API only ** */
#define ACE_SIGNALF_BUS_RESET (1<< 3) /* Expansion port /BUS_RESET signal                          ** V6 API only ** */
#define ACE_SIGNALF_NOT_READY (1<< 4) /* Expansion port READY signal                               ** V6 API only ** */
#define ACE_SIGNALF_PLUS_BUS  (1<<31) /* Expansion port is an Amstrad Plus one (read-only signal)  ** V6 API only ** */



/*
 * ** V2 API only **
 * IODATAF are field definitions to be used to set ioData in the Joystick() API
 * Warning: bits should be set to 0 (not 1!) to activate the
 *          direction and left unchanged otherwise
 */

#define ACE_IODATAF_JOYSTICK_UP    (1<<0)
#define ACE_IODATAF_JOYSTICK_DOWN  (1<<1)
#define ACE_IODATAF_JOYSTICK_LEFT  (1<<2)
#define ACE_IODATAF_JOYSTICK_RIGHT (1<<3)
#define ACE_IODATAF_JOYSTICK_FIRE2 (1<<4)
#define ACE_IODATAF_JOYSTICK_FIRE1 (1<<5)
#define ACE_IODATAF_JOYSTICK_FIRE3 (1<<6) /* Ignored on Amstrad Plus; 3rd fire button is not wired  */
#define ACE_IODATAF_JOYSTICK_PAUSE (1<<7)



/*
 * ** V7 API only **
 * HOSTMOUSEF are field definitions to be used to check buttons in the HostMouseEvent() API
 */

#define ACE_HOSTMOUSEF_BUTTON_MAIN   (1<<0)
#define ACE_HOSTMOUSEF_BUTTON_SECOND (1<<1)
#define ACE_HOSTMOUSEF_BUTTON_MIDDLE (1<<2)



/*
 * ** V7 API only **
 * HOSTGAMEPADF are field definitions to be used to check buttons in the HostGamepadEvent() API
 */

#define ACE_HOSTGAMEPADF_BUTTON_TOP              (1<< 0) /* Triangle / Y     */
#define ACE_HOSTGAMEPADF_BUTTON_RIGHT            (1<< 1) /* Circle   / B     */
#define ACE_HOSTGAMEPADF_BUTTON_BOTTOM           (1<< 2) /* X        / A     */
#define ACE_HOSTGAMEPADF_BUTTON_LEFT             (1<< 3) /* Square   / X     */
#define ACE_HOSTGAMEPADF_BUTTON_UPPER_REAR_LEFT  (1<< 4) /* L1       / LB    */
#define ACE_HOSTGAMEPADF_BUTTON_UPPER_REAR_RIGHT (1<< 5) /* R1       / RB    */
#define ACE_HOSTGAMEPADF_BUTTON_LOWER_REAR_LEFT  (1<< 6) /* L2       / LT    */
#define ACE_HOSTGAMEPADF_BUTTON_LOWER_REAR_RIGHT (1<< 7) /* R2       / RT    */
#define ACE_HOSTGAMEPADF_BUTTON_MIDDLE_LEFT      (1<< 8) /* Select   / Back  */
#define ACE_HOSTGAMEPADF_BUTTON_MIDDLE_RIGHT     (1<< 9) /* Start    / Start */
#define ACE_HOSTGAMEPADF_BUTTON_LEFT_STICK       (1<<10) /* L3       / LJ    */
#define ACE_HOSTGAMEPADF_BUTTON_RIGHT_STICK      (1<<11) /* R3       / RJ    */
#define ACE_HOSTGAMEPADF_BUTTON_MIDDLE_CENTRE    (1<<12) /* PS       / Xbox  */



/*
 * Deprecated defines
*/

#define ACE_FLAGSF_ACTIVE_LIGHT_DEVICE ACE_FLAGSF_CATCH_HOST_LIGHT_DEVICE

#define ACE_FLAGSF_EXCLUSIVE_LIGHT_DEVICE (0)
#define ACE_FLAGSF_EXCLUSIVE_MOUSE        (0)
#define ACE_FLAGSF_EXCLUSIVE_JOYSTICK     (0)
#define ACE_FLAGSF_EXCLUSIVE_PRINTER      (0)

#endif /* LIBRARIES_ACEPANSION_H */
